/*
 */

#include "sm/generic.h"
SM_RCSID("@(#)$Id: t-testmap-0.c,v 1.8 2014/06/01 00:50:21 ca Exp $")

#include "sm/error.h"
#include "sm/heap.h"
#include "sm/memops.h"
#include "sm/string.h"
#include "sm/sysexits.h"
#include "sm/test.h"
#include "sm/maps.h"
#include "sm/mapc.h"
#include "sm/map.h"
#include "sm/mapclasses.h"

#include "sm/io.h"

int Verbose = 0;

#if MTA_TEST_MAP
#define MAPC_TYPE	"tst"
#define MAPC_NAME	"test0"

static sm_ret_T
init_mapc_test(sm_maps_P *pmaps)
{
	sm_ret_T ret;

	SM_REQUIRE(pmaps != NULL);
	ret = sm_maps_init(pmaps);
	SM_TEST(*pmaps != NULL);
	if (*pmaps == NULL)
		return ret;
	SM_TEST_E(sm_is_success(ret));
	ret = sm_tst_class_create(*pmaps);
	SM_TEST_E(sm_is_success(ret));
	return ret;

  error:
	return ret;
}

static void
testh(void)
{
	sm_ret_T ret;
	sm_maps_P maps;
	sm_map_P map;
	sm_cstr_P mtype, mname;
	sm_str_P lhs, rhs;
	sm_map_key_T key;
	char *mapname, *maptype;

	maps = NULL;
	mtype = mname = NULL;
	lhs = rhs = NULL;

	maptype = MAPC_TYPE;
	mapname = MAPC_NAME;

	mtype = sm_cstr_scpyn0((const uchar *)maptype, strlen(maptype));
	SM_TEST_E(mtype != NULL);
	mname = sm_cstr_scpyn0((const uchar *)mapname, strlen(mapname));
	SM_TEST_E(mname != NULL);

	lhs = sm_str_new(NULL, 256, 1024);
	SM_TEST_E(lhs != NULL);
	rhs = sm_str_new(NULL, 256, 1024);
	SM_TEST_E(rhs != NULL);
	ret = init_mapc_test(&maps);
	SM_TEST_E(sm_is_success(ret));
	map = NULL;
	ret = sm_map_open(maps, mname, mtype, 0, "tst", SMAP_MODE_RDONLY,
			&map, SMPO_END);
	SM_TEST_E(sm_is_success(ret));

#define KEY1	"key1"
#define KEY2	"tag:key2"

	sm_str_clr(rhs);
	sm_str_assign(key, NULL, (uchar *)KEY1, strlen(KEY1), strlen(KEY1));
	ret = sm_map_lookup(map, SMMAP_FL_NONE, &key, rhs);
	SM_TEST(sm_is_success(ret));

	sm_str_clr(lhs);
	sm_str_clr(rhs);
	ret = sm_str_scat(lhs, KEY1);
	SM_TEST(sm_is_success(ret));
	ret = sm_map_rewrite(map, SMMAP_FL_NONE, lhs, rhs);
	SM_TEST(sm_is_success(ret));
	SM_TEST(strncmp(KEY1, (char *) sm_str_data(rhs), sm_str_getlen(rhs)) == 0);

	sm_str_clr(lhs);
	sm_str_clr(rhs);
	ret = sm_str_scat(lhs, KEY2);
	SM_TEST(sm_is_success(ret));
	ret = sm_map_rewrite(map, SMMAP_FL_NONE, lhs, rhs);
	SM_TEST(sm_is_success(ret));
	SM_TEST(strncmp("key2", (char *) sm_str_data(rhs), sm_str_getlen(rhs)) == 0);

#define TST_RC(key, val) do { \
	sm_str_clr(lhs);	\
	sm_str_clr(rhs);	\
	ret = sm_str_scat(lhs, key);	\
	SM_TEST(sm_is_success(ret));	\
	ret = sm_map_rewrite(map, SMMAP_FL_NONE, lhs, rhs);	\
	SM_TEST(val);	\
	} while (0)

	TST_RC("tagged:4", sm_is_temp_err(ret));
	TST_RC("tagger:5", sm_is_perm_err(ret));
	TST_RC("mytag:3", SM_MAP_DATA2BIG == ret);
	TST_RC("not:6", SM_MAP_NOTFOUND == ret);

	TST_RC("t:m4", sm_is_temp_err(ret));
	TST_RC("ta:m5", sm_is_perm_err(ret));
	TST_RC("myt:m3", SM_MAP_DATA2BIG == ret);
	TST_RC("nottt:m6", SM_MAP_NOTFOUND == ret);

	TST_RC("t1:mo", sm_is_temp_err(ret));
	TST_RC("t1:mo", sm_is_success(ret));

	TST_RC("smtpc_rcpt_conf:mn@example.org", sm_is_temp_err(ret));
	TST_RC("smtpc_rcpt_conf:mn@example.org", sm_error_perm(SM_EM_MAP, SM_E_NOMORE) == ret);
	SM_TEST(sm_str_getlen(rhs) == 0);

	TST_RC("other:mn@example.org", sm_is_temp_err(ret));
	TST_RC("other:mn@example.org", sm_error_perm(SM_EM_MAP, SM_E_NOMORE) == ret);
	SM_TEST(sm_str_getlen(rhs) == 0);

	ret = sm_map_close(map, 0);
	SM_TEST(sm_is_success(ret));

	ret = sm_maps_term(maps);
	SM_TEST(sm_is_success(ret));

	SM_CSTR_FREE(mtype);
	SM_CSTR_FREE(mname);
	SM_STR_FREE(lhs);
	SM_STR_FREE(rhs);
	return;

  error:
	sm_maps_term(maps);
}

static void
usage(const char *prg)
{
	sm_io_fprintf(smioerr, "usage: %s [options]\n"
		, prg
		);
	exit(EX_USAGE);
}

int
main(int argc, char *argv[])
{
	int c;
	char *prg;

	prg = argv[0];
	while ((c = getopt(argc, argv, "h")) != -1) {
		switch (c) {
		  default:
			usage(prg);
			/* NOTREACHED */
			return EX_USAGE;
		}
	}
	sm_test_begin(argc, argv, "testmap 0");
	testh();
	return sm_test_end();
}
#else /* MTA_TEST_MAP */
int
main(int argc, char *argv[])
{
	return 0;
}
#endif /* MTA_TEST_MAP */
